
/* i/o functions */

#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stdarg.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "io.h"
#include "config.h"

int SockOpen(const char *host, int clientPort)
{
    int sock;
    struct sockaddr_in ad;
    struct hostent *hp;

    memset(&ad, 0, sizeof(ad));
    ad.sin_family = AF_INET;

    if ((hp = gethostbyname (host)) == NULL)
        return -1;

    /* vhost support code can be added here if desired */

	/*
	 * Add a check to make sure the address has a valid IPv4 or IPv6
	 * length.  This prevents buffer spamming by a broken DNS.
	 */
	if (hp == NULL || (hp->h_length != 4 && hp->h_length != 8))
	    return -1;

	/*
	 * FIXME: make this work for multihomed hosts.
	 * We're toast if we get back multiple addresses and h_addrs[0]
	 * (aka h_addr) is not one we can actually connect to; this happens
	 * with multi-homed boxes.
	 */
	memcpy(&ad.sin_addr, hp->h_addr, hp->h_length);

    ad.sin_port = htons(clientPort);

    sock = socket(AF_INET, SOCK_STREAM, 0);
    if (sock < 0)
	return -errno;
    
    if (connect(sock, (struct sockaddr *) &ad, sizeof(ad)) < 0)
    {
	close(sock);
	return -errno;
    }
    
    return sock;
}


int SockPrintf(int sock, char *format,...)
{
    va_list ap;
    char buf[8192];

    va_start(ap, format);
#ifdef HAVE_VSNPRINTF
    vsnprintf(buf, sizeof(buf), format, ap);
#else
    vsprintf(buf, format, ap);
#endif
    va_end(ap);

    return SockWrite(sock, buf, strlen(buf));

}

int SockWrite(int sock, char *buf, int len)
{
    int n, wrlen = 0;

    while (len)
    {
	n = write(sock, buf, len);
	if (n <= 0)
	    return -1;
	len -= n;
	wrlen += n;
	buf += n;
    }
    return wrlen;
}

int SockRead(int sock, char *buf, int len)
{
    char *newline = NULL, *bp = buf;
    int n;

    if (--len < 1)
	return (-1);
    do
    {
	/* 
	 * The reason for these gymnastics is that we want two things:
	 * (1) to read \n-terminated lines,
	 * (2) to return the true length of data read, even if the
	 *     data coming in has embedded NULS.
	 */
	if ((n = recv(sock, bp, len, MSG_PEEK)) <= 0)
	{
	    if (errno == EINTR)
		continue;
	    else
		return (-1);
	}
	if ((newline = memchr(bp, '\n', n)) != NULL)
	    n = newline - bp + 1;

	if ((n = read(sock, bp, n)) == -1)
	    return (-1);

	bp += n;
	len -= n;
    }
    while
	(!newline && len);
    *bp = '\0';

    return bp - buf;
}

void AddDebug(char *cbuf)
{
#ifdef SDEBUG
    printf("%s", cbuf);
    fflush(stdout);
#endif
}
